#include "include/port.h"
C==============================================================================
C==============================================================================
	real function glref(gtype,lambda)
c       A function which accepts as input the glass type and the wavelength,
c       and returns the index of refraction of that glass at that wavelength.
	IMPLICIT_NONE
#       include "include/stdio.par"
#       include "echelle.par"
C       External Variables
c                       index into the glass type table
	integer         gtype
c                       wavelength of interest [micrometers]
	real            lambda
C       Internal Variables
C       Common Blocks
	include 'glass.cmn'
C       Executable Code
c       consistency check
	if (gtype .gt. MAXGLASS .or. gtype .lt. MINGLASS) then
	    write(STDERR,*)
     &      ' glref:  Glass type is outside of allowed range', gtype
	    write(STDERR,*)
     &      ' glref:  Range is ', MINGLASS, '<= gtype <=', MAXGLASS
	    glref = -1.
	    return
	endif
c       this is the square of the index of refraction at wavelength lambda
	glref = glc(1,gtype) + glc(2,gtype) * lambda**2 +
     &  (glc(3,gtype) + (glc(4,gtype) + (glc(5,gtype) + glc(6,gtype)/
     &  lambda**2)/lambda**2)/lambda**2)/lambda**2
	glref = sqrt(glref)
	return
	end
C==============================================================================
C==============================================================================
	integer function gldref(gtype,lambda,refn,drefdl)
c       A function which accepts as input the glass type and the wavelength.
c       It returns the index of refraction of that glass at that wavelength,
c       and the derivative of index of refraction of that glass
c       with respect to wavelength evaluated at that wavelength.
c       The function returns 0 if no errors occurs.
	IMPLICIT_NONE
#       include "include/stdio.par"
#       include "echelle.par"
C       External Variables
c                       index into the glass type table
	integer         gtype
c                       wavelength of interest [micrometers]
	real            lambda
c                       index of refraction of the glass at the wavelength
	real            refn
c                       derivative of refn wrt lambda at the wavelength
c                       [micrometers**-1]
	real            drefdl
C       Internal Variables
c                       temporary variable for computing derivative [um**-1]
	real            top
C       Common Blocks
	include 'glass.cmn'
C       External Function
C       Executable Code
c       check for consistency
	if (gtype .gt. MAXGLASS .or. gtype .lt. MINGLASS) then
	    write(STDERR,*)
     &      ' gldref:  Glass type is outside of allowed range', gtype
	    write(STDERR,*)
     &      ' gldref:  Range is ', MINGLASS, '<= gtype <=', MAXGLASS
	    gldref = -1
	    return
	endif
	gldref = 0
c       this is the square of the index of refraction at wavelength lambda
	refn = glc(1,gtype) + glc(2,gtype) * lambda**2 +
     &  (glc(3,gtype) + (glc(4,gtype) + (glc(5,gtype) + glc(6,gtype)/
     &  lambda**2)/lambda**2)/lambda**2)/lambda**2
	refn = sqrt(refn)
c       units of top are [micrometers**-1]
	top = glc(2,gtype) * lambda - (glc(3,gtype) + (2.*glc(4,gtype) +
     &  (3.*glc(5,gtype) + 4.*glc(6,gtype)/lambda**2)
     &  /lambda**2)/lambda**2)/lambda**3
c       drefdl is the {derivative of {the index of refraction} w.r.t.
c       wavelength} evaluated at wavelength lambda [micrometers**-1]
	drefdl = top / refn
	return
	end
C==============================================================================
C==============================================================================
	subroutine getglas
c       load the index of refraction coefficients from the Schott data
c       Alas, all communication is done via the common blocks.
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
#       include "echelle.par"
C       Internal Variables
	integer         i
C       Common Blocks
	include 'schott.cmn'
	include 'glass.cmn'
C       Executable Code
	if (iglass .gt. MAXGLASS .or. iglass .lt. MINGLASS)
     &  write(STDERR,*) ' WARNING: glass type outside range'
	typeg = kgnam(iglass)
	do 10 i = 0,5
	  a(i) = glc(i+1,iglass)
10      continue
	return
	end
C==============================================================================
C==============================================================================
	subroutine glassname(schott)
c       search thru the Schott data for a glass with a name matching schott.
c       Then, load up the refraction coefficients for that glass.
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
#       include "echelle.par"
C       External Variable
	character*(*)   schott
C       Internal Variables
c                       loop
	integer         i
C       Common Blocks
	include 'glass.cmn'
	include 'schott.cmn'
C       Executable Code
c       default to UBK7
	iglass = 22
c       go hunt
	do 100 i=MINGLASS,MAXGLASS
	  if (schott .eq. kgnam(i)) then
	      iglass = i
	      goto 200
	  endif
100     continue
	write(STDERR,*) 'glassname:  could not match glass ', schott
	write(STDERR,*) 'used UBK7 instead.'
200     continue
	call getglas
	return
	end
C==============================================================================
C==============================================================================
