;+ 
; NAME:
; x1dfit   
;   Version 1.3
;
; PURPOSE:
;    Fits a function to a set of x,y data
;
; CALLING SEQUENCE:
;   
;   fit = x1dfit(xdat,ydat,func=,nord=, /inter, xsize=, ysize=)
;
; INPUTS:
;   xdat       - Values along one dimension
;   ydat       - Values along the other
;
; RETURNS:
;   fit        - Values at each xdat
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   func       - String for Fitting function (POLY, LEGEND, BSPLINE,
;                GAUSS)
;   nord       - Order of the fit
;   inter      - Interactive fitting
;   xsize      - Draw window xsize (pixels)
;   ysize      - Draw window ysize (pixels)
;   sig        - Errors in the points
;   reg        - Regions of data to fit
;   LSIG       - Lower SIGMA
;   HSIG       - High SIGMA
;
; OPTIONAL OUTPUTS:
;   bset       - Bspline info
;
; COMMENTS:
;
; EXAMPLES:
;   fit = x1dfit(x, y, 'POLY', nord=5, /inter)
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  POLY_FIT
;  POLY
;  SVDFIT
;  SVLEG
;  BSPLINE
;  GAUSS
;  XGETX_PLT
;  XGETY_PLT
;  XGETXPIX_PLT
;  XGETYPIX_PLT
;  FSC_COLOR (coyote package)
;
	; REVISION HISTORY:
;   23-Nov-2001 Added BSPLIN (SB), GAUSS, REJECTION (JXP)
;   28-June-2001 Added LEGENDRE fits (JXP)
;   22-June-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;
; Common
;;;;

pro x1dfit_initcommon

;

common x1dfit_fit, svffit, fin_fit, fitprm, xfit

fitprm = { nord: 0, $
           func: '', $
           minpt: 0L, $    ; Minimum number of points to keep
           flg_rej: 0, $
           lsigma: 0., $
           hsigma: 0.}
end

;;;;
; Events
;;;;

pro x1dfit_event, ev

common x1dfit_fit

  WIDGET_CONTROL, ev.top, get_uvalue = state, /no_copy
  WIDGET_CONTROL, ev.id, get_uvalue = uval

  case uval of
      'ERRORB' : widget_control, state.error_msg_id, set_value=''
      'FUNCLIST' : begin
          nfunc = ev.index
          case nfunc of 
              0 : fitprm.func = 'POLY'
              1 : fitprm.func = 'LEGEND'
              2 : fitprm.func = 'BSPLIN'
              3 : fitprm.func = 'GAUSS'
              else:
          endcase
          ; Reset rejected
          rej = where(state.gdpix[0:state.norg-1] MOD 8 GE 4, nrej)
          if nrej NE 0 then state.gdpix[rej] = state.gdpix[rej] - 4
      end
      'UP' : begin
          fitprm.nord = fitprm.nord+1
          widget_control, state.lblordr_id, $
            set_value=string(fitprm.nord, format='(i2)')
      end
      'DOWN' : begin
          fitprm.nord = fitprm.nord-1
          widget_control, state.lblordr_id, $
            set_value=string(fitprm.nord, format='(i2)')
      end
      'DRAW_BASE' : begin
          if ev.enter EQ 0 then begin ; Turn off keyboard
              widget_control, state.text_id, sensitive = 0
          endif
          WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
          return
      end
      'DRAW' : begin
          widget_control, state.text_id, /sensitive, /input_focus ; Turn on keyb
          case ev.type of
              0 : begin ; Button press
                  case ev.press of
                      1 : begin     ; Add a Data point with weight 10
                          state.xtot[state.ntot] = xgetx_plt(ev.x,state.pos, $
                                                         state.xymnx, $
                                                         state.size)
                          state.ytot[state.ntot] = xgety_plt(ev.y,state.pos, $
                                                         state.xymnx, $
                                                         state.size)
                          state.wtot[state.ntot] = 0.1
                          state.gdpix[state.ntot] = 1
                          state.ntot = state.ntot + 1
                      end 
                      4 : begin    ; Delete/Undelete a data point
                          x1dfit_delete, state, ev.x, ev.y
                      end
                  endcase
              end
              1 : begin ; Button release
                  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                  return
              end
              2 : begin ; Motion event
                  state.xcurs = ev.x
                  state.ycurs = ev.y
                  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                  return
              end
          endcase
      end
      'TEXT' : begin
          eventch = string(ev.ch)
          if (state.flg_reg EQ 1 AND eventch NE 's') then begin
              widget_control, state.error_msg_id, $
                set_value='Expecting another s !!'
              WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
              return
          endif
          case eventch of
              'u': begin
                  fitprm.nord = fitprm.nord+1
                  widget_control, state.lblordr_id, $
                    set_value=string(fitprm.nord, format='(i2)')
              end
              'd': begin
                  fitprm.nord = (fitprm.nord-1 > 0)
                  widget_control, state.lblordr_id, $
                    set_value=string(fitprm.nord, format='(i2)')
              end
              'b': state.xymnx[1] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; bottom
              'l': state.xymnx[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; left
              'r': state.xymnx[2] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; right
              't': state.xymnx[3] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; top
              's': begin  ; Region
                  if state.flg_reg EQ 0 then begin
                      state.flg_reg = 1
                      state.reg[state.nreg,0] = xgetx_plt(state.xcurs,state.pos, $
                                                    state.xymnx, state.size)
                      WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                      return
                  endif else begin
                      state.flg_reg = 0
                      state.reg[state.nreg,1] = xgetx_plt(state.xcurs,state.pos, $
                                                    state.xymnx, state.size)
                      state.nreg = state.nreg + 1
                      x1dfit_SetReg, state
                  endelse
              end
              'x': x1dfit_DelReg, state        ; Delete one region
              'S': x1dfit_DelReg, state, /all  ; Delete all regions and reset
              'w': state.xymnx = state.svxymnx ; Reset the screen
              'W': state.xymnx = state.svxymnx ; Reset the screen
              'q': begin
                  x1dfit_setpnt, state
                  widget_control, ev.top, /destroy
                  return
              end
              else:  ; Nothing
          endcase
      end
;            REJECTION
      'HSIGVAL': begin
          fitprm.hsigma = ev.value
          x1dfit_UnDel, state
          if ev.value NE 0. then fitprm.flg_rej = 1 else begin
              if fitprm.lsigma EQ 0. then fitprm.flg_rej = 0
          endelse
      end
      'LSIGVAL': begin
          fitprm.lsigma = ev.value
          x1dfit_UnDel, state
          if ev.value NE 0. then fitprm.flg_rej = 1 else begin
              if fitprm.hsigma EQ 0. then fitprm.flg_rej = 0
          endelse
      end
      'DONE' : begin
          x1dfit_setpnt, state
          widget_control, ev.top, /destroy
          return
      end
      else:
  endcase

  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
; Update Fit
  x1dfit_UpdateFit, ev.top
; Update Plot
  x1dfit_UpdatePlot, ev.top
end
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;
;  Plot
;;;;;;;;;;;;;;;;;;;;


pro x1dfit_UpdatePlot, base_id
  
common x1dfit_fit

; State

  widget_control, base_id, get_uvalue=state, /no_copy

; Plot Data

  widget_control, state.draw_id, get_value=wind
  wset, wind

  color = getcolor(/load)

;  ORIGINAL DATA NOT IN REGION
  gdorg = where(state.gdpix[0:state.norg-1] EQ 1, count) ; Original data 
  if count NE 0 then $
    plot, state.xdat[gdorg], state.ydat[gdorg], psym=1, $
    position=state.pos, xrange=[state.xymnx[0],state.xymnx[2]], $
    yrange=[state.xymnx[1],state.xymnx[3]], xstyle=1, ystyle=1, $
    background=color.black, color=color.white $
  else $
    plot, [-1.0],  [state.svxymnx[3]+1.], psym=1, $
    position=state.pos, xrange=[state.xymnx[0],state.xymnx[2]], $
    yrange=[state.xymnx[1],state.xymnx[3]], xstyle=1, ystyle=1, $
    background=color.black, color=color.white

  for q=0,7 do begin
      if q EQ 1 then continue
      pts = where(state.gdpix[0:state.norg-1] EQ q, count)
      if count NE 0 then begin
          case q of
              0: oplot, [state.xdat[pts]], [state.ydat[pts]], psym=2, $
                color=color.red  ; DELETED DATA OUT OF REGION
              2: oplot, [state.xdat[pts]], [state.ydat[pts]], psym=2, $
                color=color.purple  ; DELETED DATA IN REGION
              3: oplot, [state.xdat[pts]], [state.ydat[pts]], psym=1, $
                color=color.navy  ; GOOD DATA IN REGION
              4: 
              5: oplot, [state.xdat[pts]], [state.ydat[pts]], psym=4, $
                color=color.orange  ; REJECTED DATA OUT OF REGION
              6: 
              7: oplot, [state.xdat[pts]], [state.ydat[pts]], psym=4, $
                color=color.yellow  ; REJECTED DATA OUT OF REGION
              else:
          endcase
      endif
  endfor

      
;    Added points
  if(state.ntot GT state.norg) then begin
      oplot, extrac(state.xtot,state.norg,state.ntot-state.norg), $
        extrac(state.ytot,state.norg,state.ntot-state.norg), psym=5, $
        color=color.cyan
  endif

;    Fit
  oplot, xfit, fin_fit, color=color.red ; Best fit Red  

; Return State

  widget_control, base_id, set_uvalue=state, /no_copy

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;
;  Fit
;;;;;;;;;;;;;;;;;;;;

pro x1dfit_UpdateFit, base_id

common x1dfit_fit

; State

  widget_control, base_id, get_uvalue=state, /no_copy

; Set Good points

  if state.nreg EQ 0 then $
    gdpt = where(state.gdpix[0:state.ntot-1] EQ 1, count) $
  else begin
      if state.norg LT state.ntot then begin   ; Deal with added points
          gdpt = where(state.gdpix[0:state.norg-1] EQ 3, count) 
          for i=state.norg, state.ntot-1 do gdpt = [gdpt,i]
          count = count + state.ntot - state.norg
      endif else gdpt = where(state.gdpix[0:state.norg-1] EQ 3, count)
  endelse

  mask = lonarr(state.ntot)
  mask[gdpt] = 1
  fitprm.minpt = fitprm.minpt < count/2

; Catch error

  if count EQ 0 then begin
      widget_control, state.error_msg_id, $
        set_value='No good data!  Adjust regions as necessary...'
      widget_control, base_id, set_uvalue=state, /no_copy
      return
  endif

; Error

  if fitprm.nord EQ 0 then begin
      widget_control, state.error_msg_id, set_value='FIT requires nord > 0'
      fitprm.nord = 1
      widget_control, state.lblordr_id, $
        set_value=string(fitprm.nord, format='(i2)')
  endif else begin  ; FIT!
      delvarx, svffit

      if fitprm.flg_rej EQ 0 then begin
          fit = x_fit(state.xtot[0:state.ntot-1], state.ytot[0:state.ntot-1], $
                      fitprm.func, fitprm.nord, $
                      MSK=mask,$
                      SIG=state.wtot[0:state.ntot-1], FFIT=svffit)
      endif else begin ; FIT with REJECTION!
          rejpt = -1
          fit = x_fitrej(state.xtot[0:state.ntot-1], $
                         state.ytot[0:state.ntot-1], $
                         fitprm.func, fitprm.nord, $
                         MSK=mask,$
                         SIG=state.wtot[0:state.ntot-1], $
                         LSIGMA=fitprm.lsigma, $
                         HSIGMA=fitprm.hsigma, $
                         REJPT=rejpt, $
                         MINPT=fitprm.minpt, $
                         FFIT=svffit)
          if rejpt[0] NE -1 then state.gdpix[rejpt] = state.gdpix[rejpt]+4
      endelse
  endelse


  fin_fit = x_calcfit(xfit, fitprm.func, svffit, NORD=fitprm.nord)

  delvarx, mask, fit

  widget_control, base_id, set_uvalue=state, /no_copy

end

;;;;;;;;;;;;;;;;;;;;
;  Reset
;;;;;;;;;;;;;;;;;;;;

pro x1dfit_Reset, base_id

; State

  widget_control, base_id, get_uvalue=state, /no_copy

;
  state.xtot[0:n_elements(state.xdat)-1] = state.xdat
  state.ytot[0:n_elements(state.xdat)-1] = state.ydat
  if state.flg_wgt EQ 0 then state.wtot[0:n_elements(state.xdat)-1] = 1.0 $
  else state.wtot[0:n_elements(state.xdat)-1] = state.sig

  state.ntot = n_elements(state.xdat)


; gdpix Flag
;   1 = Good
;   2 = In region
;   4 = Rejected by fit
  state.gdpix[0:n_elements(state.xdat)-1] = 1

; Regions

  if state.nreg NE 0 then begin
      for i=0,state.nreg-1 do begin
          newpix = where(state.xdat LE state.reg[i,1] AND $
                         state.xdat GE state.reg[i,0], count)
          if count NE 0 then $
            state.gdpix[newpix] = state.gdpix[newpix] + 2 ; Bitwise flag
      endfor
  endif


; Plotting

  state.xymnx = state.svxymnx

;

  widget_control, base_id, set_uvalue=state, /no_copy

end

;;;;;;;;;;;;;;;;;;;;
;  Delete
;;;;;;;;;;;;;;;;;;;;

pro x1dfit_Delete, state, xp, yp

; Deletes the nearest data point in pixel space

  dist = sqrt( (xgetxpix_plt(state.xtot[0:state.ntot-1],state.pos,state.xymnx, $
                            state.size) - xp)^2 + $
               (xgetypix_plt(state.ytot[0:state.ntot-1],state.pos,state.xymnx, $
                            state.size) - yp)^2 )

; Minimum distance
  mdist = min(dist, jmin)

  if jmin LT state.norg then begin
      if state.gdpix[jmin] MOD 2 EQ 0 then $
        state.gdpix[jmin] = state.gdpix[jmin] + 1 $  ; Add it back
      else state.gdpix[jmin] = state.gdpix[jmin] - 1  ; Delete it
  endif else begin  ; Added data point
      if state.ntot EQ state.norg + 1 then state.ntot = state.norg $
      else begin
          state.gdpix[jmin] = 0 ; Data point deleted!
          gdadd = where(state.gdpix[state.norg:state.ntot-1] NE 0)
          gdadd = gdadd + state.norg
          state.ntot = state.ntot - 1
          state.xtot[state.norg:state.ntot-1] = state.xtot[gdadd]
          state.ytot[state.norg:state.ntot-1] = state.ytot[gdadd]
          state.wtot[state.norg:state.ntot-1] = state.wtot[gdadd]
          state.gdpix[state.norg:state.ntot-1] = state.gdpix[gdadd]
      endelse
  endelse

end

;;;;;;;;;;;;;;;;;;;;
;  UnDel
;;;;;;;;;;;;;;;;;;;;

pro x1dfit_UnDel, state

; Deletes the nearest data point in pixel space

  del = where(state.gdpix EQ 0, ndel)
  if ndel NE 0 then state.gdpix[del] = 1

end

;;;;
;  Set Regions
;;;;

pro x1dfit_SetReg, state

;  Determines all points in the regions -- Only examines data

;  Require reg[1] > reg[0]

  if state.reg[state.nreg-1,1] LT state.reg[state.nreg-1,0] then begin
      tmp = state.reg[state.nreg-1,1]
      state.reg[state.nreg-1,1] = state.reg[state.nreg-1,0]
      state.reg[state.nreg-1,0] = tmp
  endif

; Require regions dont overlap

  if state.nreg GT 1 then begin
      bla = where(state.reg[0:state.nreg-1,*] LT state.reg[state.nreg-1,1] AND $
                  state.reg[0:state.nreg-1,*] GT state.reg[state.nreg-1,0], count)
      if count NE 0 then begin
          widget_control, state.error_msg_id, set_value='Regions may not overlap!'
          state.nreg = state.nreg - 1
          return
      endif
  endif
                  

; Adjust new pixels

  newpix = where(state.xdat LE state.reg[state.nreg-1,1] AND $
                 state.xdat GE state.reg[state.nreg-1,0] AND $
                 state.gdpix MOD 4 LT 2, count)
  if count NE 0 then begin
      state.gdpix[newpix] = state.gdpix[newpix] + 2 ; Bitwise
;     Reset Rejected pixels
      rej = where(state.gdpix[newpix] MOD 8 GE 4, count)
      if count NE 0 then $
        state.gdpix[newpix[rej]] = state.gdpix[newpix[rej]] - 4 ; Bitwise
  endif else begin
      widget_control, state.error_msg_id, $
        set_value='Region must contain at least 1 new data point'
      state.nreg = state.nreg - 1
  endelse

end
  
;;;;
; Delete Region(s)
;;;;

pro x1dfit_DelReg, state, ALL=all

  if state.nreg EQ 0 then return
; Deletes 1 or All regions

  if (keyword_set (ALL) OR state.nreg EQ 1) then begin
      if state.nreg EQ 0 then return $
      else begin
          regpix = where(state.gdpix[0:state.norg-1] MOD 4 GT 1)
          state.gdpix[regpix] = state.gdpix[regpix] - 2
          state.nreg = 0
      endelse
  endif else begin
      xpos = xgetx_plt(state.xcurs, state.pos, state.xymnx, state.size)
      fndreg = where( xpos GE state.reg[0:state.nreg-1,0] AND $
                      xpos LE state.reg[0:state.nreg-1,1], count)
      newreg = where(NOT ( xpos GE state.reg[0:state.nreg-1,0] AND $
                      xpos LE state.reg[0:state.nreg-1,1]), nnew)
;                    complement=newreg, ncomplement=nnew)
      if count EQ 0 then begin      ; Not in the region
          widget_control, state.error_msg_id, set_value='No region found!'
          return
      endif
; Reset gdpix
      fndpix = where(state.xdat LE state.reg[fndreg,1] AND $
                     state.xdat GE state.reg[fndreg,0])
      state.gdpix[fndpix] = state.gdpix[fndpix] - 2
; Reset regions
      for i=0,nnew-1 do begin  
          state.reg[i,0] = state.reg[newreg[i],0]
          state.reg[i,1] = state.reg[newreg[i],1]
      endfor
      state.nreg = state.nreg - 1
  endelse
end

;;;;;;;;;;;;;;;;;;;;

pro x1dfit_setpnt, state

; 

  *state.pnt_nreg = state.nreg
  *state.pnt_reg = state.reg

return
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x1dfit, xin, yin, FUNC=func, NORD=nord, INTER=INTER, $
                 XSIZE=xsize, YSIZE=ysize, SIG=sig, REG=reg, FFIT=ffit,$
                 REJ=rej, LSIG=lsig, HSIG=hsig, MINPT=minpt

common x1dfit_fit


;
  if  N_params() LT 1  then begin 
    print,'Syntax - ' + $
             'fit = x1dfit([xdat], ydat, FUNC=, NORD=, /INTER, '
    print, '        XSIZE=,YSIZE=, SIG=, REG=, FFIT='
    print, '        REJ=, LSIG=, HSIG=, MINPT=) [V1.2]'
    return, -1
  endif 

; Initialize the common block
  x1dfit_initcommon


; Set xdat, ydat

  if keyword_set( yin ) then begin
      if n_elements(xin) NE n_elements(yin) then begin
          print, 'Wrong array sizes!'
          return, -1
      endif
      xdat = xin
      ydat = yin
  endif else begin
      xdat = findgen( n_elements(xin) )
      ydat = xin
  endelse

;  Optional Keywords

  if not keyword_set( NORD ) then fitprm.nord = 3 else fitprm.nord = nord
  if not keyword_set( FUNC ) then fitprm.func = 'POLY' else fitprm.func=func
  if keyword_set( REJ ) then begin
      fitprm.lsigma = 3.
      fitprm.hsigma = 3.
      fitprm.flg_rej = 1
  endif

;   REJECTION
  if keyword_set(LSIG) OR keyword_set( HSIG ) then begin
      fitprm.flg_rej = 1
      if keyword_set( LSIG ) then fitprm.lsigma = lsig
      if keyword_set( HSIG ) then fitprm.hsigma = hsig
  endif
  if not keyword_set( MINPT ) then fitprm.minpt = long(0.5*n_elements(xin))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Non-interactive

  if not keyword_set (INTER) then begin

      if fitprm.flg_rej EQ 0 then $
        return, x_fit(xdat, ydat, $
                      fitprm.func, fitprm.nord, $
                      SIG=sig, REG=reg, FFIT=ffit) $
      else $
        return, x_fitrej(xdat, ydat, fitprm.func, fitprm.nord, $
                         SIG=sig, REG=reg, $
                         LSIGMA=fitprm.lsigma, $
                         HSIGMA=fitprm.hsigma, $
                         MINPT=fitprm.minpt, $
                         FFIT=ffit) 

  endif else begin
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INTERACTIVE

  if not keyword_set( XSIZE ) then    xsize = 1000
  if not keyword_set( YSIZE ) then    ysize = 600

; Device

  device, decompose=1

;    Pointer for Final fit
  tmpreg = fltarr(100,2)
  pnt_reg = PTR_NEW( tmpreg )
  nreg = 0
  pnt_nreg = PTR_NEW( nreg )
  
;    STATE
  state = { $
            xdat: xdat, $
            ydat: ydat, $
            sig: fltarr(n_elements(ydat)), $
            norg: n_elements(xdat), $
            pnt_nreg: pnt_nreg, $
            pnt_reg: pnt_reg, $
            xtot: fltarr(200000), $ ; Fitting vectors
            ytot: fltarr(200000), $
            wtot: fltarr(200000), $ ; Weighting Factor
            flg_wgt: 0, $       ; 0 = No input 1 = Input
            lsigma: 0., $
            hsigma: 0., $
            ntot: n_elements(xdat), $
            nreg: 0, $
            reg: tmpreg, $
            flg_reg: 0, $
            pos: [0.1,0.1,0.95,0.95], $ ; Plotting
            svxymnx: [min(xdat)-0.01*abs(max(xdat)-min(xdat)), $
                      min(ydat)-0.01*abs(max(ydat)-min(ydat)), $
                      max(xdat)+0.01*abs(max(xdat)-min(xdat)), $
                      max(ydat)+0.01*abs(max(ydat)-min(ydat))], $
            xymnx: fltarr(4), $
            size: intarr(2), $
            xcurs: 0.0, $
            ycurs: 0.0, $
            base_id: 0L, $      ; Widgets
            lblordr_id: 0L, $
            draw_base_id: 0L, $
            draw_id: 0L, $
            text_id: 0L, $
            funclist_id: 0L, $
            error_msg_id: 0L, $
            reject_base_id: 0L, $
            hsig_id: 0L, $
            lsig_id: 0L, $
            gdpix: intarr(200000) $
          }
; XFIT
  
  xfit = findgen(5*state.norg)*(state.svxymnx[2]-state.svxymnx[0])/ $
    float(5*state.norg)  + state.svxymnx[0] 

;   SIG

  if keyword_set( SIG ) then state.sig = sig

;   REG

  if keyword_set( REG ) then begin
      sz = size(reg, /dimensions)
      state.reg[0:sz[0]-1,*] = reg
      state.nreg = sz[0]
  endif
      
;    WIDGET
  base = WIDGET_BASE( title = 'x1dfit: Interactive Mode', /column)
  state.base_id = base
  
;      Toolbar
          
  toolbar = WIDGET_BASE( state.base_id, /row, /frame, /base_align_center,$
                           /align_center)
;        Version 
  strlbl = strarr(10)
  strlbl = ['x1dfit', ' ', 'Ver 1.1']
  verslabel = WIDGET_TEXT(toolbar, value=strlbl, xsize=15, ysize=3)
;        Function
  funcval = ['POLY', 'LEGEND', 'BSPLIN', 'GAUSS']
  state.funclist_id = WIDGET_LIST(toolbar, VALUE=funcval, uvalue='FUNCLIST', $
                                  ysize=4)
  case strtrim(fitprm.func) of 
      'POLY' :  widget_control, state.funclist_id, set_list_select=0
      'LEGEND' :  widget_control, state.funclist_id, set_list_select=1
      'BSPLIN' :  widget_control, state.funclist_id, set_list_select=2
      'GAUSS' :  widget_control, state.funclist_id, set_list_select=3
  endcase
  
;        Order      
  ordrbase = WIDGET_BASE(toolbar, /column)
  upordr = WIDGET_BUTTON(ordrbase, value='Up',uvalue='UP')
  tmp_string = string(fitprm.nord, format='(i2)')
  state.lblordr_id = WIDGET_LABEL(ordrbase, value=tmp_string, $
                                  /dynamic_resize, /align_center)
  dwnordr = WIDGET_BUTTON(ordrbase, value='Down',uvalue='DOWN')
  
;        Rejection
  state.reject_base_id = widget_base(toolbar, /column, /align_center,$
                                     /frame)
  hsigbase = widget_base(state.reject_base_id, /row, /align_right)
  state.hsig_id = cw_field(hsigbase, title='High Sig', $
                           value=fitprm.hsigma, $
                           xsize=5, ysize=1, /return_events, $
                           uvalue='HSIGVAL')
  lsigbase = widget_base(state.reject_base_id, /row, /align_right)
  state.lsig_id = cw_field(lsigbase, title='Low Sig', $
                           value=fitprm.lsigma, $
                           xsize=5, ysize=1, /return_events, $
                           uvalue='LSIGVAL')
;        Help
  strhelp = strarr(50)
  strhelp = ['   Help Menu   ',$
             'l -- Set Left Window',$
             'r -- Set Right Window',$
             'b -- Set Bottom Window',$
             't -- Set Top Window',$
             'w -- Reset screen',$
             's,s -- Set Region',$
             'x -- Remove one Region',$
             'S -- Remove all Regions',$
             'q -- Quit and save',$
             'u -- Increase order 1',$
             'd -- Decrease order 1']
  help_text_id = widget_text(toolbar, value=strhelp, xsize=30, ysize=4,$
                             /scroll)
;      Error base
  error_base_id  = widget_base(toolbar, /column)
  error_btn_id = widget_button(error_base_id, value='Erase Error Msg', $
                               uvalue='ERRORB')
;          Error Messages
  state.error_msg_id = widget_text(error_base_id, value='', xsize=40, $
                                   ysize=2, /scroll)
  
;      Drawing
  state.size[0] = xsize
  state.size[1] = ysize
  state.draw_base_id = widget_base(base, /column, /base_align_left, $
                                   uvalue='DRAW_BASE', frame=2, $
                                   /tracking_events)
  
  state.draw_id = widget_draw(state.draw_base_id, xsize=state.size[0], $
                              ysize=state.size[1], /frame, $
                              /button_events, /motion_events, uvalue='DRAW')
;      Text
  state.text_id = widget_text(base, $
                              /all_events, $
                              scr_xsize = 1, $
                              scr_ysize = 1, $
                              units = 0, $
                              uvalue = 'TEXT', $
                              value = '')
;      Done
  done = WIDGET_BUTTON(toolbar, value='Done',uvalue='DONE', /align_right)
  
; Realize
  WIDGET_CONTROL, base, /realize
  
; Include all elements at first
  state.gdpix[0:n_elements(xdat)] = 1
  
; Color Table 

; Update
  WIDGET_CONTROL, base, set_uvalue = state, /no_copy
  x1dfit_Reset, base
  x1dfit_UpdateFit, base
  x1dfit_UpdatePlot, base
  
; Send to the xmanager
  xmanager, 'x1Dfit', base
  
; Passing back
  
;    Fit parameters
  
  hsig = fitprm.hsigma
  lsig = fitprm.lsigma
  func = fitprm.func
  nord = fitprm.nord
      
;    Regions
  if arg_present( REG ) then begin
      nreg = *pnt_nreg
      reg = *pnt_reg
      delvarx, tmpreg
      if nreg GT 0 then tmpreg = reg[0:nreg-1,*] $
      else begin
          tmpreg = fltarr(1,2)
          tmpreg[0,0] = xdat[0]
          tmpreg[0,1] = xdat[n_elements(xdat)-1]
      endelse
      delvarx, reg
      reg = temporary(tmpreg)
  endif 

;  End game

  if arg_present( FFIT ) then ffit = svffit

;  Reset fin_fit to the xin points

  return, x_calcfit(xin, fitprm.func, svffit, NORD=fitprm.nord)

  endelse

  return, -1
end
