;+ 
; NAME:
; x_calcfit   
;   Version 1.0
;
; PURPOSE:
;    Calculates a fit-function at a set of x points
;
; CALLING SEQUENCE:
;   
;   fit = x_calcfit(xdat,func,ffit, NORD=)
;
; INPUTS:
;   xdat       - Values along one dimension
;   func       - String for Fitting function (POLY, LEGEND, BSPLIN,
;                GAUSS)
;   ffit       - Output from the fitting stuff
;
; RETURNS:
;   fit        - Values at each xdat
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   nord       - Required for LEGEND
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   fit = x_calcfit(x, 'POLY', 5)
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  POLY_FIT
;  POLY
;  SVDFIT
;  SVLEG
;  BSPLIN
;  GAUSS
;
	; REVISION HISTORY:
;   20-Nov-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x_calcfit, xdat, func, ffit, NORD=nord

;
  if  N_params() LT 3  then begin 
    print,'Syntax - ' + $
             'fit = x_calcfit(xdat, func, ffit, NORD=) (V1.0)'
    return, -1
  endif 

;  Optional Keywords

  if (func EQ 'LEGEND' AND not keyword_set( NORD )) then begin
      print, 'Nord must be set for LEGEND'
      return, -1
  endif

; Function

  case func of 
      'POLY': return, poly(xdat,ffit)
      'LEGEND': begin
          npnt = n_elements(xdat)
          fit = fltarr(npnt)
          for i=0L,npnt-1 do fit[i] = $
            total(ffit * call_function('svdleg', xdat[i], nord)) 
          return, fit
      end
      'BSPLIN': return, bspline_valu(xdat, ffit)
      'GAUSS': begin
          case n_elements(ffit) of 
              3: fit = ffit[0]*exp(- ((ffit[1]-xdat)/ffit[2])^2 / 2)
              4: fit = ffit[0]*exp(- ((ffit[1]-xdat)/ffit[2])^2 / 2) $
                + ffit[3]
              5: fit = ffit[0]*exp(- ((ffit[1]-xdat)/ffit[2])^2 / 2) $
                + ffit[3] + ffit[4]*xdat
              6: fit = ffit[0]*exp(- ((ffit[1]-xdat)/ffit[2])^2 / 2) $
                + ffit[3] + ffit[4]*xdat + ffit[5]*xdat^2
              else: begin
                  print, 'In correct number of ffit elements'
                  return, -1
              end
          endcase
          return, fit
      end
;  No function set
      else: begin
          print, 'Invalid function declaration! '
          return, -1
      end
  endcase

end
