;+ 
; NAME:
; x_fit   
;   Version 1.1
;
; PURPOSE:
;    Fits a function to a set of x,y data
;
; CALLING SEQUENCE:
;   
;   fit = x_fit(xdat,ydat,func,nord,sig=,reg=) 
;
; INPUTS:
;   xdat       - Values along one dimension
;   ydat       - Values along the other
;   func       - String for Fitting function (POLY, LEGEND, BSPLIN,
;                GAUSS)
;   nord       - Order of the fit
;
; RETURNS:
;   fit        - Values at each xdat
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   inter      - Interactive fitting
;   sig        - Errors in the points
;   reg        - Regions of data to fit
;   msk        - Mask  (0 = Do NOT include)
;   guess      - First guess for GAUSS routine
;
; OPTIONAL OUTPUTS:
;   ffit     - Functional form
;
; COMMENTS:
;
; EXAMPLES:
;   fit = x_fit(x, y, 'POLY', nord=5)
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  POLY_FIT
;  POLY
;  SVDFIT
;  SVLEG
;
	; REVISION HISTORY:
;   20-Nov-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x_fit, xdat, ydat, func, nord, SIG=sig, REG=reg, FFIT=ffit, $
                MSK=msk, GUESS=guess

;
  if  N_params() LT 3  then begin 
    print,'Syntax - ' + $
             'fit = x_fit(xdat, ydat, func, [nord], '
    print, '        SIG=, REG=, FFIT=, MSK=msk, GUESS=)'
    return, -1
  endif 

;  Optional Keywords

  igood = lindgen(n_elements(xdat))

; Mask
  if keyword_set( MSK ) then igood = where(msk NE 0)

;   Pre-set Regions
  if keyword_set( REG ) then begin
      regpt = x_fndreg(xdat[igood], reg, NPNT=npnt)  ; Find the regions 
      if npnt EQ 0 then return, -1
      igood = temporary(igood[regpt])
  endif 


  xfit = xdat(igood)
  yfit = ydat(igood)
  if keyword_set( SIG ) then sfit = sig(igood)
                                
; Function

  case func of 
      'POLY': begin
          ffit = poly_fit(xfit,yfit,nord, measure_errors=sfit,$
                          /double, status=status)
          if status EQ 0 then return, poly(xdat,ffit) $
          else begin
              print, 'POLYFIT Failed!', status
              return, -1
          endelse
      end
      'LEGEND': begin
          if not keyword_set( REG ) then begin
              ffit = svdfit(xdat,ydat,nord, measure_errors=sfit,$
                            /double, /legendre, yfit=ans)
              return, ans
          endif else begin
              ffit = svdfit(xdat, ydat, nord, /double, /legendre, $
                             singular=singular)
              if(singular EQ 0) then begin
                  fit = fltarr(n_elements(xdat))
                  for i=0,npnt-1 do fit[i] = $
                    total(ffit * call_function('svdleg', xdat[i], nord)) 
                  return, fit
              endif else begin
                  print, 'Problem with svdfit: Order too high?', singular
                  return, -1
              endelse
          endelse
      end
      'BSPLIN': begin
          ffit= bspline_iterfit(xfit, yfit, $
                                  nbkpts=nord, yfit = bsplinefit)
          if (size(ffit,/tname) EQ "INT") then begin
              print, 'Problem with bspline_fit: Order is probably too high'
              return, -1 
          endif else return, bspline_valu(xdat, ffit)
      end
      'GAUSS': begin
          if keyword_set( NORD ) then nord = 3 > nord < 6
          fit= gaussfit(xfit, yfit, ffit, ESTIMATE=guess, $
                                  NTERMS=nord)
      end
;  No function set
      else: begin
          print, 'Invalid function declaration! '
          return, -1
      end
  endcase

end
