;+ 
; NAME:
; x_fitrej
;   Version 1.0
;
; PURPOSE:
;    Fits a function to a set of x,y data with rejection!
;
; CALLING SEQUENCE:
;   
;   fit =
;   x_fitrej(xdat,ydat,func=,nord=,sig=,reg=,
;            lsigma=,hsigma=,maxrej=,niter=,minpt=)
;
; INPUTS:
;   xdat       - Values along one dimension
;   ydat       - Values along the other
;
; RETURNS:
;   fit        - Values at each xdat
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   func       - String for Fitting function (POLY, LEGEND, BSPLIN)
;   nord       - Order of the fit
;   inter      - Interactive fitting
;   sig        - Errors in the points
;   reg        - Regions of data to fit
;   lsigma     - Lower sigma threshold
;   hsigma     - Upper sigma threshold
;   maxrej     - Maximum points to reject per iteration
;   niter      - Number of iterations for rejection
;   minpt      - Minimum # of points
;
; OPTIONAL OUTPUTS:
;   bsplin     - Bspline structure
;
; COMMENTS:
;
; EXAMPLES:
;   fit = x_fitrej(x, y, 'POLY', nord=5)
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  POLY_FIT
;  POLY
;  SVDFIT
;  SVLEG
;  DJS_REJECT
;
; REVISION HISTORY:
;   20-Nov-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x_fitrej, xdat, ydat, func, nord, SIG=sig, REG=reg, $
                   LSIGMA=lsigma, HSIGMA=hsigma, MAXREJ=maxrej, $
                   NITER=niter, MINPT=minpt, FFIT=ffit, REJPT=rejpt, $
                   MSK=msk

;
  if  N_params() LT 4  then begin 
    print,'Syntax - ' + $
             'fit = x_fitrej(xdat, ydat, func, nord, '
    print, '        [SIG=, REG=, LSIGMA=, HSIGMA=, MAXREJ=, NITER=, MINPT=,'
    print, '        REJPT=, FFIT=, MSK=])  [v1.0]'
    return, -1
  endif 

;  Optional Keywords

  if (not keyword_set( LSIGMA ) AND not keyword_set( HSIGMA )) then return, -1
  if not keyword_set( NITER ) then niter = 9999
  if not keyword_set( MINPT ) then minpt = 1
  if not keyword_set( MAXREJ ) then maxrej = 100


  igood = lindgen(n_elements(xdat))

; Mask
  if keyword_set( MSK ) then igood = where(msk NE 0)

;   Pre-set Regions
  if keyword_set( REG ) then begin
      regpt = x_fndreg(xdat[igood], reg, NPNT=npnt)  ; Find the regions 
      if npnt EQ 0 then return, -1
      igood = temporary(igood[regpt])
  endif 

  xfit = xdat[igood]
  yfit = ydat[igood]
  if keyword_set( SIG ) then sfit = sig[igood]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN LOOP
  
;  First Fit
  fit = x_fit(xfit, yfit, func, nord, SIG=sfit, FFIT=ffit)

  iter = 0L
  while iter LT niter do begin
      
;   Reject  

      tmpmax = (n_elements(xfit) - minpt) < maxrej
      qmod = djs_reject(yfit, fit, outmask=outmask, upper=hsigma, $
                        lower=lsigma, maxrej=tmpmax, /sticky)

;   Check number of good points
      tmp = where(outmask EQ 1, ntmp)
      if (qmod EQ 1 OR ntmp LE minpt) then iter = niter else iter = iter + 1

;   FIT
      fit = x_fit(xfit, yfit, func, nord, SIG=sfit, FFIT=ffit, MSK=outmask)
      if n_elements(fit) EQ 1 then return, -1

  endwhile

;  Calculate fit everywhere

  fit = x_calcfit(xdat, func, ffit, NORD=nord)

; Rejected points
  if arg_present( REJPT ) then begin
      rej = where(outmask EQ 0, nrej)
      if nrej EQ 0 then rejpt = -1 else rejpt = igood[rej]
  endif

; Return
  return, fit

end
